@@ -1,5 +1,5 @@
-#![doc(html_root_url = "https://docs.rs/exmex/0.5.0")]
-//! Exmex is an **ex**tendable **m**athematical **ex**pression evaluator.  
+#![doc(html_root_url = "https://docs.rs/exmex/0.6.0")]
+//! Exmex is a fast **ex**tendable **m**athematical **ex**pression evaluator.  
 //! ```rust
 //! # use std::error::Error;
 //! # fn main() -> Result<(), Box<dyn Error>> {
@@ -16,7 +16,7 @@
 //!
 //! ## Variables
 //! For variables we can use curly brackets as shown in the following expression.
-//! Variables' values are passed as slices to [`eval`](Expression::eval).
+//! Variables' values are passed as slices to [`eval`](FlatEx::eval).
 //! ```rust
 //! # use std::error::Error;
 //! # fn main() -> Result<(), Box<dyn Error>> {
@@ -122,14 +122,13 @@ mod operators;
 mod parse;
 mod util;
 
-pub use expression::{Expression, Node, BinOpVec};
-
-pub use util::UnaryOpVec;
+pub use expression::{FlatEx};
 
 pub use parse::{parse, parse_with_default_ops, parse_with_number_pattern, ExParseError};
 
 pub use operators::{make_default_operators, BinOp, Operator};
 
+
 /// Parses a string, evaluates a string, and returns the resulting number.
 ///
 /// # Errrors
@@ -138,8 +137,8 @@ pub use operators::{make_default_operators, BinOp, Operator};
 /// [`ExParseError`](ExParseError) is returned.
 ///
 pub fn eval_str(text: &str) -> Result<f64, ExParseError> {
-    let expr = parse_with_default_ops(text)?;
-    Ok(expr.eval(&vec![]))
+    let flat_ex = parse_with_default_ops(text)?;
+    Ok(flat_ex.eval(&vec![]))
 }
 
 #[cfg(test)]
@@ -158,31 +157,38 @@ mod tests {
 
     #[test]
     fn test_readme() {
-        fn readme() -> Result<f32, ExParseError> {
+        fn readme() -> Result<f64, ExParseError> {
             let result = eval_str("sin(73)")?;
             assert_float_eq_f64(result, 73f64.sin());
             let expr = parse_with_default_ops::<f64>("2*{x}^3-4/{z}")?;
-
             let value = expr.eval(&[5.3, 0.5]);
             assert_float_eq_f64(value, 289.75399999999996);
+            Ok(value)
+        }
+        fn readme_int() -> Result<u32, ExParseError> {
             let ops = vec![
                 Operator {
-                    repr: "invert",
-                    bin_op: None,
-                    unary_op: Some(|a: f32| 1.0 / a),
+                    repr: "|",
+                    bin_op: Some(BinOp {
+                        op: |a: u32, b: u32| a | b,
+                        prio: 0,
+                    }),
+                    unary_op: None,
                 },
                 Operator {
-                    repr: "sqrt",
+                    repr: "!",
                     bin_op: None,
-                    unary_op: Some(|a: f32| a.sqrt()),
+                    unary_op: Some(|a: u32| !a),
                 },
             ];
-            let expr = parse::<f32>("sqrt(invert({a}))", ops)?;
-            let result = expr.eval(&[0.25]);
-            assert_float_eq_f32(result, 2.0);
+            let expr = parse::<u32>("!({a}|{b})", ops)?;
+            let result = expr.eval(&[0, 1]);
+            assert_eq!(result, u32::MAX - 1);
             Ok(result)
         }
         assert!(!readme().is_err());
+        assert!(!readme_int().is_err());
+        
     }
     #[test]
     fn test_variables() {
@@ -322,6 +328,10 @@ mod tests {
             eval_str(&"-(-1+((-3.14159265358979)/5)*2)").unwrap(),
             2.256637061435916,
         );
+        assert_float_eq_f64(
+            eval_str(&"((2-4)/5)*2").unwrap(),
+            -0.8,
+        );
         assert_float_eq_f64(
             eval_str(&"-(-1+(sin(-3.14159265358979)/5)*2)").unwrap(),
             1.0,
