@@ -1,6 +1,6 @@
-use crate::expression::{BinOpVec, Expression, Node};
+use crate::expression::{BinOpVec, Expression, FlatEx, Node};
 use crate::operators::{make_default_operators, BinOp, Operator};
-use crate::util::{UnaryOpVec, apply_unary_ops};
+use crate::util::{apply_unary_ops, CompositionOfUnaryOps};
 use itertools::{izip, Itertools};
 use num::Float;
 use regex::{Regex, RegexSet};
@@ -162,7 +162,7 @@ where
 fn make_expression<T>(
     parsed_tokens: &[ParsedToken<T>],
     parsed_vars: &[String],
-    unary_ops: UnaryOpVec<T>,
+    unary_ops: CompositionOfUnaryOps<T>,
 ) -> Result<(Expression<T>, usize), ExParseError>
 where
     T: Copy + FromStr + Debug,
@@ -203,7 +203,7 @@ where
                     .take_while(|uo_| uo_.is_some())
                     .flatten(),
             )
-            .collect::<UnaryOpVec<_>>();
+            .collect::<CompositionOfUnaryOps<_>>();
         let n_uops = uops.len();
 
         match &parsed_tokens[i + n_uops] {
@@ -218,7 +218,11 @@ where
                 }
             },
             ParsedToken::Var(name) => {
-                let expr = Expression::new(vec![Node::Var(find_var_index(&name))], BinOpVec::new(), uops)?;
+                let expr = Expression::new(
+                    vec![Node::Var(find_var_index(&name))],
+                    BinOpVec::new(),
+                    uops,
+                )?;
                 Ok((Node::Expr(expr), n_uops + 1))
             }
             ParsedToken::Num(n) => Ok((Node::Num(apply_unary_ops(&uops, *n)), n_uops + 1)),
@@ -260,7 +264,7 @@ where
                             ParsedToken::Paren(p) => match p {
                                 Paren::Open => {
                                     let msg = "This is probably a bug An opening paren cannot be the predecessor of a binary opertor";
-                                    panic!(msg);
+                                    panic!("{}", msg);
                                 }
                                 Paren::Close => {
                                     bin_ops.push(unpack_binop(b.bin_op));
@@ -287,8 +291,11 @@ where
             ParsedToken::Paren(p) => match p {
                 Paren::Open => {
                     idx_tkn += 1;
-                    let (expr, i_forward) =
-                        make_expression::<T>(&parsed_tokens[idx_tkn..], &parsed_vars, UnaryOpVec::new())?;
+                    let (expr, i_forward) = make_expression::<T>(
+                        &parsed_tokens[idx_tkn..],
+                        &parsed_vars,
+                        CompositionOfUnaryOps::new(),
+                    )?;
                     nodes.push(Node::Expr(expr));
                     idx_tkn += i_forward;
                 }
@@ -423,7 +430,7 @@ where
 ///
 /// An error is returned in case [`parse_with_number_pattern`](parse_with_number_pattern)
 /// returns one.
-pub fn parse<'a, T>(text: &str, ops: Vec<Operator<'a, T>>) -> Result<Expression<T>, ExParseError>
+pub fn parse<'a, T>(text: &str, ops: Vec<Operator<'a, T>>) -> Result<FlatEx<T>, ExParseError>
 where
     <T as std::str::FromStr>::Err: Debug,
     T: Copy + FromStr + Debug,
@@ -445,9 +452,9 @@ where
 /// * the argument `text` contained a character that did not match any regex (e.g.,
 ///   if there is a `Δ` in `text` but no [operator](Operator) with
 ///   [`repr`](Operator::repr) equal to `Δ` is given),
-// 
+//
 // from check_preconditions
-// 
+//
 /// * the to-be-parsed string is empty,
 /// * a number or variable is next to another one, e.g., `2 {x}`,
 /// * wlog a number or variable is on the right of a closing parenthesis, e.g., `)5`,
@@ -456,9 +463,9 @@ where
 /// * too many closing parentheses at some position, e.g., `(4+6) - 5)*2`,
 /// * the last element is an operator, e.g., `1+`,
 /// * the number of opening and closing parenthesis do not match, e.g., `((4-2)`,
-// 
+//
 // from make_expression
-// 
+//
 /// * in `parsed_tokens` a closing parentheses is directly following an operator, e.g., `+)`, or
 /// * a unary operator is followed directly by a binary operator, e.g., `sin*`.
 ///
@@ -466,7 +473,7 @@ pub fn parse_with_number_pattern<'a, T>(
     text: &str,
     ops: Vec<Operator<'a, T>>,
     number_regex_pattern: &str,
-) -> Result<Expression<T>, ExParseError>
+) -> Result<FlatEx<T>, ExParseError>
 where
     <T as std::str::FromStr>::Err: Debug,
     T: Copy + FromStr + Debug,
@@ -481,8 +488,12 @@ where
         .unique()
         .collect::<Vec<_>>();
     check_preconditions(&parsed_tokens[..])?;
-    let (expr, _) = make_expression(&parsed_tokens[0..], &parsed_vars, UnaryOpVec::new())?;
-    Ok(expr)
+    let (expr, _) = make_expression(
+        &parsed_tokens[0..],
+        &parsed_vars,
+        CompositionOfUnaryOps::new(),
+    )?;
+    Ok(expr.flatten())
 }
 
 /// Parses a string into an expression that can be evaluated using default operators.
@@ -491,12 +502,13 @@ where
 ///
 /// An error is returned in case [`parse`](parse)
 /// returns one.
-pub fn parse_with_default_ops<T>(text: &str) -> Result<Expression<T>, ExParseError>
+pub fn parse_with_default_ops<T>(text: &str) -> Result<FlatEx<T>, ExParseError>
 where
     <T as std::str::FromStr>::Err: Debug,
     T: Float + FromStr + Debug,
 {
     let ops = make_default_operators::<T>();
+    // println!("{:#?}", ops);
     Ok(parse(&text, ops)?)
 }
 
