@@ -18,7 +18,7 @@ pub fn assert_float_eq_f64(f1: f64, f2: f64) {
 }
 
 /// Container of unary operators of one expression
-pub type UnaryOpVec<T> = SmallVec<[fn(T) -> T; 8]>;
+pub type CompositionOfUnaryOps<T> = SmallVec<[fn(T) -> T; 8]>;
 
 
 /// Applies unary operators one after the other starting with the last.
@@ -27,7 +27,7 @@ pub type UnaryOpVec<T> = SmallVec<[fn(T) -> T; 8]>;
 /// * `uops` - unary operators to be applied
 /// * `x` - number the unary operators are applied to
 ///
-pub fn apply_unary_ops<T>(uops: &UnaryOpVec<T>, x: T) -> T {
+pub fn apply_unary_ops<T>(uops: &CompositionOfUnaryOps<T>, x: T) -> T {
     let mut result = x;
     // rev, since the last uop is applied first by convention
     for uo in uops.iter().rev() {
